/**
* \file twi.c
* @brief gibt Zugriff auf die I2C Schnittstelle des Mikrocontrollers
* \author Lars Neubert
* \date Feb 2010
*/

#include "twi.h"

volatile uint8_t twi_timeout=0;

/** 
* Initialisiert die I2C Schnittstelle des Mikrocontrollers
*/
void twi_init(void)						// TWBR soll zwischen 10 und 255 sein
{
	#if TWI_INT_PULLUP==1
	// SCL und SDA als Eingang definieren + PullUp aktivieren
	TWI_DDR &= ~((1<<TWI_SCL)|(1<<TWI_SDA));
	TWI_PORT |= (1<<TWI_SCL)|(1<<TWI_SDA);
	#endif

	TWSR = 0; 							// kein prescaler, Statusbits 0 setzen
	// TWPS1	TWPS0	prescaler
	//     0		0		1
	//     0		1		4
	//     1		0		16
	//     1		1		64
	//TWSR=(1<<TWPS0);					// prescaler = 4
	TWBR = ((F_CPU/TWI_CLK)-16)/2;		// TWBR=(F_CPU/scl_clk-16)/(2*4^TWPS) --> scl_clk=F_CPU/(16+2*TWBR*4^TWPS) 									 
	// bei "warning: large integer implicitly truncated to unsigned type" ist TWI_CLK unpassend gewhlt
	// andere Frequenz bzw. Prescaler whlen!!

}

/** 
* Sendet eine START-Kondition mit anschlieendem Senden der Adresse und der Datenrichtung
* @param address 7Bit Adresse des Slave
* @param rw legt Datenrichtung fest (\ref TWI_READ 1, \ref TWI_WRITE 0, siehe twi.h)
* @return gibt an, ob ein Timeout ausgelst wurde (=1 Timeout und somit Fehler, =0 alles i.O.)
*/
uint8_t twi_start(uint8_t address, uint8_t rw)
{
	TWCR=(1<<TWINT)|(1<<TWSTA)|(1<<TWEN);	// sende START-Kondition

	twi_timeout=0;
	while (!(TWCR & (1<<TWINT))){			// w8 dass START bertragen wurden
		#if TWI_TIMEOUT_ON==1
		#if TWI_TIMEOUT_ISR==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>TWI_TIMEOUT_BOUND) {return true;}
		#endif		
	};	
	//if ((TWSR & 0xF8)!=TWI_START) && (TWSR & 0xF8)!=TWI_REP_STAR) {ERROR();}
	TWDR=(address<<1)+rw;
	TWCR=(1<<TWINT)|(1<<TWEN);				// sende Slave-adresss + RWcommand
	twi_timeout=0;
	while (!(TWCR & (1<<TWINT))){			// w8 dass Adresse bertragen wurden		
		#if TWI_TIMEOUT_ON==1
		#if TWI_TIMEOUT_ISR==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>TWI_TIMEOUT_BOUND) {return true;}
		#endif
	}
	//if ((TWSR & 0xF8)!=TWI_MTX_ADR_ACK) && (TWSR & 0xF8)!=TWI_MRX_ADR_ACK) {ERROR();}
	return false;
}

/** 
* Sendet ein Datenbyte (vorherrige Angabe der Datenrichtung \ref TWI_WRITE bei START-Kondition, siehe \ref twi_start)
* @param data zu sendendes Datenbyte
* @return gibt an, ob ein Timeout ausgelst wurde (=1 Timeout und somit Fehler, =0 alles i.O.)
*/
uint8_t twi_write(uint8_t data)
{
	TWDR = data;							// data in TWDR-Register laden
	TWCR = (1<<TWINT)|(1<<TWEN);			// starte bertargung von DATA
	twi_timeout=0;
	while (!(TWCR & (1<<TWINT))){};			// w8 dass Daten gesendet
		#if TWI_TIMEOUT_ON==1
		#if TWI_TIMEOUT_ISR==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>TWI_TIMEOUT_BOUND) {return true;}
		#endif
	//if ((TWSR & 0xF8)!=TWI_MTX_DATA_ACK) {ERROR();}
	return false;
}

/** 
* Empfngt ein Datenbyte ohne senden eines Acknowledgement (vorherrige Angabe der Datenrichtung \ref TWI_READ bei START-Kondition, siehe \ref twi_start)
* @param erg Pointer auf Speicherbereich in den das empfangene Datenbyte abgelegt werden soll
* @return gibt an, ob ein Timeout ausgelst wurde (=1 Timeout und somit Fehler, =0 alles i.O.)
*/
uint8_t twi_read_nack(uint8_t *erg)					// Lesen und NACK senden
{
	TWCR = (1<<TWINT)|(1<<TWEN);
	twi_timeout=0;
	while(!(TWCR & (1<<TWINT))){
		#if TWI_TIMEOUT_ON==1
		#if TWI_TIMEOUT_ISR==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>TWI_TIMEOUT_BOUND) {return true;}
		#endif	
	};
	//if ((TWSR & 0xF8)!=TWI_MRX_DATA_NACK) {ERROR();}
	*erg=TWDR;
	return false;
}

/** 
* Empfngt ein Datenbyte und Besttigt dieses durch Acknowledgement (vorherrige Angabe der Datenrichtung \ref TWI_READ bei START-Kondition, siehe \ref twi_start)
* @param erg Pointer auf Speicherbereich in den das empfangene Datenbyte abgelegt werden soll
* @return gibt an, ob ein Timeout ausgelst wurde (=1 Timeout und somit Fehler, =0 alles i.O.)
*/
uint8_t twi_read_ack(uint8_t *erg)					// Lesen und ACK senden
{
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);
	twi_timeout=0;
	while(!(TWCR & (1<<TWINT))){
		#if TWI_TIMEOUT_ON==1
		#if TWI_TIMEOUT_ISR==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>TWI_TIMEOUT_BOUND) {return true;}
		#endif
	}
	//if ((TWSR & 0xF8)!=TWI_MRX_DATA_ACK) {ERROR();}
	*erg=TWDR;
	return false;
}

/*
uint8_t twi_read(uint8_t ack_nack)			// Lesen und ACK/NACK senden
{
	if (ack_nack==1){	//ACK
		TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);
		twi_timeout=0;
#if TWI_TIMEOUT_ON==1
		while (!(TWCR & (1<<TWINT))&&(twi_timeout<TWI_TIMEOUT_BOUND)){};
#else
		while (!(TWCR & (1<<TWINT))){};
#endif
		//if ((TWSR & 0xF8)!=TWI_MRX_DATA_ACK) {ERROR();}
	}
	else {	//NACK
		TWCR = (1<<TWINT)|(1<<TWEN);
		twi_timeout=0;
#if TWI_TIMEOUT_ON==1
		while (!(TWCR & (1<<TWINT))&&(twi_timeout<TWI_TIMEOUT_BOUND)){};
#else
		while (!(TWCR & (1<<TWINT))){};
#endif
		//if ((TWSR & 0xF8)!=TWI_MRX_DATA_NACK) {ERROR();}
	}
	return TWDR;
}
*/

/** 
* Sendet eine STOP-Kondition (Freigabe des Busses)
*/
void twi_stop(void)
{
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);	// sende STOP-Kondition
}
