/**
* \file twi.h
* @brief Konfigurationsdatei fr die I2C Schnittstelle des Mikrocontrollers
* \author Lars Neubert
* \date Feb 2010
*/
#ifndef TWI_H
#define TWI_H

#include <avr/io.h>
#include <util/delay.h>

/**
* definiert Mikrocontroller-Taktfrequenz falls diese nicht im Makefile definiert wurde
*/
#ifndef F_CPU
	#define F_CPU 16000000UL           
#endif

/**
* definiert TWI-Taktfrequenz
*/
#ifndef TWI_CLK
	#define TWI_CLK 400000UL           
#endif

/**
* gibt an, ob mit den internen PullUp Widerstnden an SCL / SDA des AVR 
* <br> gearbeiten werden soll ( =1 PullUp akt., =0 PullUp deakt.) 
* <br><B> Bei externen PullUp Widerstnden an SCL / SDA - unbedinkt deaktivieren! </B>
*/
#define TWI_INT_PULLUP 1

#if TWI_INT_PULLUP==1
	#ifdef __AVR_ATmega8__
		#define TWI_DDR		DDRC	/**< Datenrichtungsregister zum aktivieren der internen Pullups der SDA / SCL Leitung */
		#define TWI_PORT	PORTC	/**< Portregister zum aktivieren der internen Pullups der SDA / SCL Leitung */
		#define TWI_SDA		PC4		/**< Pin der SDA Leitung zum aktivieren der internen Pullups*/
		#define TWI_SCL		PC5		/**< Pin der SCL Leitung zum aktivieren der internen Pullups*/
	#endif

	#ifdef __AVR_ATmega32__
		#define TWI_DDR		DDRC 	/**< Datenrichtungsregister zum aktivieren der internen Pullups der SDA / SCL Leitung */
		#define TWI_PORT	PORTC	/**< Portregister zum aktivieren der internen Pullups der SDA / SCL Leitung */
		#define TWI_SDA		PC1		/**< Pin der SDA Leitung zum aktivieren der internen Pullups*/
		#define TWI_SCL		PC0		/**< Pin der SCL Leitung zum aktivieren der internen Pullups*/
	#endif

	#ifdef __AVR_AT90CAN128__

		#define TWI_DDR		DDRD	/**< Datenrichtungsregister zum aktivieren der internen Pullups der SDA / SCL Leitung */
		#define TWI_PORT	PORTD	/**< Portregister zum aktivieren der internen Pullups der SDA / SCL Leitung */
		#define TWI_SDA		PC1		/**< Pin der SDA Leitung zum aktivieren der internen Pullups*/
		#define TWI_SCL		PC0		/**< Pin der SCL Leitung zum aktivieren der internen Pullups*/
	#endif

	#ifdef __AVR_ATmega1281__
		#define TWI_DDR		DDRD	/**< Datenrichtungsregister zum aktivieren der internen Pullups der SDA / SCL Leitung */
		#define TWI_PORT	PORTD	/**< Portregister zum aktivieren der internen Pullups der SDA / SCL Leitung */
		#define TWI_SDA		PD1		/**< Pin der SDA Leitung zum aktivieren der internen Pullups*/
		#define TWI_SCL		PD0		/**< Pin der SCL Leitung zum aktivieren der internen Pullups*/
	#endif
#endif

/**
* dient zur Angabe der Datenrichtung 'Lesen' bei \ref twi_start (siehe \ref twi.c)
*/
#define TWI_READ    1	// als Parameter fr twi_start

/**
* dient zur Angabe der Datenrichtung 'Schreiben' bei \ref twi_start (siehe \ref twi.c)
*/
#define TWI_WRITE   0	// als Parameter fr twi_start

/**
* gibt an, ob mit einem Timeout gearbeitet werden soll (=1 ja, =0 nein)
*/
#define TWI_TIMEOUT_ON 1    // mit twi timeout arbieten = 1, ohen twi timeout arbeiten = 0	

/**
* gibt an, ob mit bei dem Timeout mit einer Timer Interruptroutine gearbeitet werden soll (=1 ja, =0 nein)
*/
#define TWI_TIMEOUT_ISR 0

/**
* dient als Vergleichswert zur Variablen \ref twi_timeout fr das Arbeiten mit einem Timeout
* <br> Dies sollte abhngig von der jeweiligen Timer-Interruptroutine eingestellt werden und so bemessen sein, dass ca 300ms bis zum auslsen eines Timeout verstreichen.
* <br> (Zeit=300ms=\ref TWI_TIMEOUT_BOUND*'Zeit bis zum auslsen der Timer-Interruptroutine'; Bsp. ist Timer-Interruptroutine auf 100ms eingestellt, so sollte \ref TWI_TIMEOUT_BOUND 3 sein).
* <br> Ist \ref TWI_TIMEOUT_ISR = 0, so wird nicht mit einer Interuptroutine gearbeitet und \ref TWI_TIMEOUT_BOUND gibt direkt die Zeit in Millisekunden an (zwischen 1..254).
*/
#define TWI_TIMEOUT_BOUND    250

#define true       1
#define false      0

/**
* globale Variable zum Zhlen der Timer-Interrupts
*/
extern volatile uint8_t twi_timeout;

extern void twi_init(void);
extern uint8_t twi_start(uint8_t address, uint8_t rw);
extern uint8_t twi_write(uint8_t data);
extern uint8_t twi_read_nack(uint8_t *erg);
extern uint8_t twi_read_ack(uint8_t *erg);
extern void twi_stop(void);

/*
// ***************************************************************************
// TWI State codes 
// von http://www.mikrocontroller.net/topic/87597#new
// ***************************************************************************
// General TWI Master staus codes                      
#define TWI_START					0x08  // START has been transmitted  
#define TWI_REP_START				0x10  // Repeated START has been transmitted
#define TWI_ARB_LOST				0x38  // Arbitration lost

// TWI Master Transmitter staus codes                      
#define TWI_MTX_ADR_ACK				0x18  // SLA+W has been tramsmitted and ACK received
#define TWI_MTX_ADR_NACK			0x20  // SLA+W has been tramsmitted and NACK received 
#define TWI_MTX_DATA_ACK			0x28  // Data byte has been tramsmitted and ACK received
#define TWI_MTX_DATA_NACK			0x30  // Data byte has been tramsmitted and NACK received 

// TWI Master Receiver staus codes  
#define TWI_MRX_ADR_ACK				0x40  // SLA+R has been tramsmitted and ACK received
#define TWI_MRX_ADR_NACK			0x48  // SLA+R has been tramsmitted and NACK received
#define TWI_MRX_DATA_ACK			0x50  // Data byte has been received and ACK tramsmitted
#define TWI_MRX_DATA_NACK			0x58  // Data byte has been received and NACK tramsmitted

// TWI Slave Transmitter staus codes
#define TWI_STX_ADR_ACK				0xA8  // Own SLA+R has been received; ACK has been returned
#define TWI_STX_ADR_ACK_M_ARB_LOST	0xB0  // Arbitration lost in SLA+R/W as Master; own SLA+R has been received; ACK has been returned
#define TWI_STX_DATA_ACK			0xB8  // Data byte in TWDR has been transmitted; ACK has been received
#define TWI_STX_DATA_NACK			0xC0  // Data byte in TWDR has been transmitted; NOT ACK has been received
#define TWI_STX_DATA_ACK_LAST_BYTE	0xC8  // Last data byte in TWDR has been transmitted (TWEA = "0"); ACK has been received

// TWI Slave Receiver staus codes
#define TWI_SRX_ADR_ACK				0x60  // Own SLA+W has been received ACK has been returned
#define TWI_SRX_ADR_ACK_M_ARB_LOST	0x68  // Arbitration lost in SLA+R/W as Master; own SLA+W has been received; ACK has been returned
#define TWI_SRX_GEN_ACK				0x70  // General call address has been received; ACK has been returned
#define TWI_SRX_GEN_ACK_M_ARB_LOST	0x78  // Arbitration lost in SLA+R/W as Master; General call address has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_ACK		0x80  // Previously addressed with own SLA+W; data has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_NACK		0x88  // Previously addressed with own SLA+W; data has been received; NOT ACK has been returned
#define TWI_SRX_GEN_DATA_ACK		0x90  // Previously addressed with general call; data has been received; ACK has been returned
#define TWI_SRX_GEN_DATA_NACK		0x98  // Previously addressed with general call; data has been received; NOT ACK has been returned
#define TWI_SRX_STOP_RESTART		0xA0  // A STOP condition or repeated START condition has been received while still addressed as Slave

// TWI Miscellaneous status codes
#define TWI_NO_STATE				0xF8  // No relevant state information available; TWINT = "0"
#define TWI_BUS_ERROR				0x00  // Bus error due to an illegal START or STOP condition
*/

#endif
